@@ -36,56 +36,95 @@ from pyAgrum.causal._types import NameSet
 def getCausalModel(cm: csl.CausalModel, size=None) -> str:
   """
   return a HTML representing the causal model
-  :param cm: the causal model
-  :param size: passd
-  :param vals:
-  :return:
+
+  Parameters
+  ----------
+  cm: CausalModel
+    the causal model
+  size: int|str
+    the size of the rendered graph
+
+  Returns
+  -------
+  pydot.Dot
+    the dot representation
   """
-  return gnb.getDot(cm.toDot())
+  if size is None:
+    size=pyAgrum.config['causal', 'default_graph_size']
+  return gnb.getDot(cm.toDot(),size)
 
 
-def showCausalModel(cm: csl.CausalModel, size: str = "4"):
+def showCausalModel(cm: csl.CausalModel, size= None):
   """
-  Shows a graphviz svg representation of the causal DAG ``d``
+  Shows a pydot svg representation of the causal DAG
+
+  Parameters
+  ----------
+  cm: CausalModel
+    the causal model
+  size: int|str
+    the size of the rendered graph
   """
-  gnb.showDot(cm.toDot())
+  if size is None:
+    size=pyAgrum.config['causal', 'default_graph_size']
+  gnb.showDot(cm.toDot(),size=size)
 
 
 def getCausalImpact(model: csl.CausalModel, on: Union[str, NameSet], doing: Union[str, NameSet],
-                    knowing: Optional[NameSet] = None, values: Optional[Dict[str, int]] = None) -> Tuple[
-        str, "pyAgrum.Potential", str]:
+                    knowing: Optional[NameSet] = None, values: Optional[Dict[str, int]] = None):
   """
   return a HTML representing of the three values defining a causal impact : formula, value, explanation
-  :param model: the causal model
-  :param on: the impacted variable(s)
-  :param doing: the variable(s) of intervention
-  :param knowing: the variable(s) of evidence
-  :param values : values for certain variables
 
-  :return: a triplet (CausalFormula representation (string), pyAgrum.Potential, explanation)
+  Parameters
+  ----------
+  model: CausalModel
+    the causal model
+  on: str | Set[str]
+    the impacted variable(s)
+  doing: str | Set[str]
+    the interventions
+  knowing: str | Set[str]
+    the observations
+  values: Dict[str,int] default=None
+    value for certain variables
+
+  Returns
+  -------
+  HTML
   """
-  formula, impact, explanation = csl.causalImpact(
-      model, on, doing, knowing, values)
-  return gnb.getSideBySide(getCausalModel(model),
-                           "?" if formula is None else (
-                               '$$\\begin{equation*}' + formula.toLatex() + '\\end{equation*}$$'),
-                           "No result" if formula is None else impact,
-                           captions=["Causal Model", "Explanation : " + explanation,
-                                     "Impact : $" + ("?" if formula is None else formula.latexQuery(values)) + "$"])
+  formula, impact, explanation = csl.causalImpact(model, on, doing, knowing, values)
+
+  gnb.flow.clear()
+  gnb.flow.add(getCausalModel(model),caption="Causal Model")
+  if formula is None:
+    gnb.flow.add(explanation,caption="Impossible")
+  else:
+    gnb.flow.add('$$\\begin{equation*}' + formula.toLatex() + '\\end{equation*}$$',caption="Explanation : "+explanation )
+  gnb.flow.add("No result" if formula is None else impact,caption="Impact") # : $" + ("?" if formula is None else formula.latexQuery(values)) + "$")
+
+  return gnb.flow.html()
 
 
 def showCausalImpact(model: csl.CausalModel, on: Union[str, NameSet], doing: Union[str, NameSet],
                      knowing: Optional[NameSet] = None, values: Optional[Dict[str, int]] = None):
   """
   display a HTML representing of the three values defining a causal impact :  formula, value, explanation
-  :param model: the causal model
-  :param on: the impacted variable(s)
-  :param doing: the variable(s) of intervention
-  :param knowing: the variable(s) of evidence
-  :param values : values for certain variables
+
+  Parameters
+  ----------
+  model: CausalModel
+    the causal model
+  on: str | Set[str]
+    the impacted variable(s)
+  doing: str | Set[str]
+    the interventions
+  knowing: str | Set[str]
+    the observations
+  values: Dict[str,int] default=None
+    value for certain variables
   """
   html = getCausalImpact(model, on, doing, knowing, values)
-  IPython.display.display(IPython.display.HTML(html))
+  IPython.display.display(html)
 
 
 csl.CausalModel._repr_html_ = lambda self: gnb.getDot(
