@@ -1,7 +1,9 @@
-#!/usr/bin/python
-# -*- coding: utf-8 -*-
+"""
+The purpose of this module is to provide tools for mapping credal Network (and inference) in dot language in order to
+be displayed/saved as image.
+"""
 
-# (c) Copyright by Pierre-Henri Wuillemin, UPMC, 2017
+# (c) Copyright by Pierre-Henri Wuillemin, 2022
 # (pierre-henri.wuillemin@lip6.fr)
 
 # Permission to use, copy, modify, and distribute this
@@ -22,24 +24,22 @@
 # ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
 # OR PERFORMANCE OF THIS SOFTWARE!
 
-from __future__ import print_function
-
 import time
-import math
 import hashlib
 
-import matplotlib.pyplot as plt
-import numpy as np
-import pyAgrum as gum
-import pydotplus as dot
-import shutil
+from tempfile import mkdtemp
 
-from .proba_histogram import saveFigProbaMinMax
+import matplotlib.pyplot as plt
+import pydot as dot
+
+import pyAgrum as gum
+import pyAgrum.lib._colors as gumcols
+from pyAgrum.lib.proba_histogram import saveFigProbaMinMax
 
 
 def CN2dot(cn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmapNode=None, cmapArc=None, showMsg=None):
   """
-  create a pydotplus representation of the BN
+  create a pydot representation of the BN
 
   Parameters
   ----------
@@ -57,8 +57,6 @@ def CN2dot(cn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmapNode
       color map to show the vals of Nodes
     cmapArc: ColorMap
       color map to show the vals of Arcs
-    dag : pyAgrum.DAG
-      only shows nodes that have their id in the dag (and not in the whole BN)
     showMsg: dict
       a nodeMap of values to be shown as tooltip
 
@@ -73,6 +71,10 @@ def CN2dot(cn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmapNode
   if cmapArc is None:
     cmapArc = plt.get_cmap(gum.config["notebook", "default_arc_cmap"])
 
+  # default
+  minarcs = 0
+  maxarcs = 100
+
   if arcWidth is not None:
     minarcs = min(arcWidth.values())
     maxarcs = max(arcWidth.values())
@@ -85,10 +87,9 @@ def CN2dot(cn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmapNode
       fgcol = gum.config["credalnet", "default_node_fgcolor"]
       res = ""
     else:
-      bgcol = gum._proba2bgcolor(nodeColor[n], cmapNode)
-      fgcol = gum._proba2fgcolor(nodeColor[n], cmapNode)
-      res = " : {0:2.5f}".format(
-        nodeColor[n] if showMsg is None else showMsg[n])
+      bgcol = gumcols.proba2bgcolor(nodeColor[n], cmapNode)
+      fgcol = gumcols.proba2fgcolor(nodeColor[n], cmapNode)
+      res = f" : {nodeColor[n] if showMsg is None else showMsg[n]:2.5f}"
 
     node = dot.Node('"' + n + '"', style="filled",
                     shape="polygon",
@@ -96,7 +97,8 @@ def CN2dot(cn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmapNode
                     peripheries="1",
                     fillcolor=bgcol,
                     fontcolor=fgcol,
-                    tooltip='"({0}) {1}{2}"'.format(bn.idFromName(n), n, res))
+                    tooltip=f'"({bn.idFromName(n)}) {n}{res}"'
+                    )
     graph.add_node(node)
 
   for a in bn.arcs():
@@ -114,33 +116,38 @@ def CN2dot(cn, size=None, nodeColor=None, arcWidth=None, arcColor=None, cmapNode
         pw = 1
         av = 1
     if arcColor is None:
-      col = gum.getBlackInTheme()
+      col = gumcols.getBlackInTheme()
     else:
       if a in arcColor:
-        col = gum._proba2color(arcColor[a], cmapArc)
+        col = gumcols.proba2color(arcColor[a], cmapArc)
       else:
-        col = gum.getBlackInTheme()
+        col = gumcols.getBlackInTheme()
 
     edge = dot.Edge('"' + bn.variable(a[0]).name() + '"', '"' + bn.variable(a[1]).name() + '"',
                     penwidth=pw, color=col,
-                    tooltip="{} : {}".format(a, av))
+                    tooltip=f"{a} : {av}"
+                    )
     graph.add_edge(edge)
 
   if size is None:
     size = gum.config["notebook", "default_graph_size"]
+
+  # dynamic member makes pylink unhappy
+  # pylint: disable=no-member
   graph.set_size(size)
   return graph
 
 
-def CNinference2dot(cn, size=None, engine=None, evs={}, targets={}, nodeColor=None, arcWidth=None, arcColor=None,
-                    cmapNode=None, cmapArc=None, dag=None):
+def CNinference2dot(cn, size=None, engine=None, evs=None, targets=None, nodeColor=None, arcWidth=None, arcColor=None,
+                    cmapNode=None, cmapArc=None, dag=None
+                    ):
   """
-  create a pydotplus representation of an inference in a BN
+  create a pydot representation of an inference in a BN
 
   Parameters
   ----------
-    cn : pyAgrum.BayesNet
-      the Bayesian network
+    cn : pyAgrum.CredalNet
+      the credal network
     size: str
       size of the rendered graph
     engine:  pyAgrum.Inference
@@ -166,6 +173,10 @@ def CNinference2dot(cn, size=None, engine=None, evs={}, targets={}, nodeColor=No
   -------
     the desired representation of the inference
   """
+  if evs is None:
+    evs = {}
+  if targets is None:
+    targets = {}
   bn = cn.current_bn()
   if cmapNode is None:
     cmapNode = plt.get_cmap(gum.config["notebook", "default_node_cmap"])
@@ -173,32 +184,35 @@ def CNinference2dot(cn, size=None, engine=None, evs={}, targets={}, nodeColor=No
   if cmapArc is None:
     cmapArc = plt.get_cmap(gum.config["notebook", "default_arc_cmap"])
 
+  # by default
+  minarcs = 0
+  maxarcs = 100
+
   if arcWidth is not None:
     minarcs = min(arcWidth.values())
     maxarcs = max(arcWidth.values())
 
   startTime = time.time()
   if engine is None:
-    ie = gum.CNMonteCarloSampling(bn)
+    ie = gum.CNMonteCarloSampling(cn)
   else:
     ie = engine
   # ie.setEvidence(evs)
   ie.makeInference()
   stopTime = time.time()
 
-  from tempfile import mkdtemp
   temp_dir = mkdtemp("", "tmp", None)  # with TemporaryDirectory() as temp_dir:
 
   dotstr = "digraph structs {\n  fontcolor=\"" + \
-           gum.getBlackInTheme() + "\";bgcolor=\"transparent\";"
+           gumcols.getBlackInTheme() + "\";bgcolor=\"transparent\";"
 
-  if gum.config["notebook", "show_inference_time"]:
-    dotstr += "  label=\"Inference in {:6.2f}ms\";\n".format(1000 * (stopTime - startTime))
+  if gum.config["notebook", "show_inference_time"]=="True":
+    dotstr += f"  label=\"Inference in {1000 * (stopTime - startTime):6.2f}ms\";\n"
 
   dotstr += '  node [fillcolor="' + gum.config["notebook", "default_node_bgcolor"] + \
             '", style=filled,color="' + \
             gum.config["notebook", "default_node_fgcolor"] + '"];' + "\n"
-  dotstr += '  edge [color="' + gum.getBlackInTheme() + '"];' + "\n"
+  dotstr += '  edge [color="' + gumcols.getBlackInTheme() + '"];' + "\n"
 
   showdag = bn.dag() if dag is None else dag
   for nid in showdag.nodes():
@@ -212,28 +226,25 @@ def CNinference2dot(cn, size=None, engine=None, evs={}, targets={}, nodeColor=No
 
     if nodeColor is not None:
       if name in nodeColor or nid in nodeColor:
-        bgcol = gum._proba2bgcolor(nodeColor[name], cmapNode)
-        fgcol = gum._proba2fgcolor(nodeColor[name], cmapNode)
+        bgcol = gumcols.proba2bgcolor(nodeColor[name], cmapNode)
+        fgcol = gumcols.proba2fgcolor(nodeColor[name], cmapNode)
 
     # 'hard' colour for evidence (?)
     if name in evs or nid in evs:
       bgcol = gum.config["notebook", "evidence_bgcolor"]
       fgcol = gum.config["notebook", "evidence_fgcolor"]
 
-    colorattribute = 'fillcolor="{}", fontcolor="{}", color="#000000"'.format(
-      bgcol, fgcol)
+    colorattribute = f'fillcolor="{bgcol}", fontcolor="{fgcol}", color="#000000"'
     if len(targets) == 0 or name in targets or nid in targets:
       filename = temp_dir + \
                  hashlib.md5(name.encode()).hexdigest() + "." + \
                  gum.config["notebook", "graph_format"]
-      saveFigProbaMinMax(ie.marginalMin(name), ie.marginalMax(name), filename, bgcol=bgcol)
-      dotstr += ' "{0}" [shape=rectangle,image="{1}",label="", {2}];\n'.format(
-        name, filename, colorattribute)
+      saveFigProbaMinMax(ie.marginalMin(name), ie.marginalMax(name), filename, bgcolor=bgcol)
+      dotstr += f' "{name}" [shape=rectangle,image="{filename}",label="", {colorattribute}];\n'
     else:
-      dotstr += ' "{0}" [shape=polygon,sides=7,peripheries=1,{1}]'.format(name, colorattribute)
+      dotstr += f' "{name}" [shape=polygon,sides=7,peripheries=1,{colorattribute}]'
 
-  for a in showdag.arcs():
-    (n, j) = a
+  for n, j in showdag.arcs():
     if arcWidth is None:
       pw = 1
       av = ""
@@ -242,25 +253,28 @@ def CNinference2dot(cn, size=None, engine=None, evs={}, targets={}, nodeColor=No
         if maxarcs == minarcs:
           pw = 1
         else:
-          pw = 0.1 + 5 * (arcWidth[a] - minarcs) / (maxarcs - minarcs)
-        av = arcWidth[a]
+          pw = 0.1 + 5 * (arcWidth[n, j] - minarcs) / (maxarcs - minarcs)
+        av = arcWidth[n, j]
       else:
         pw = 1
         av = ""
 
     if arcColor is None:
-      col = gum.getBlackInTheme()
+      col = gumcols.getBlackInTheme()
     else:
-      if a in arcColor:
-        col = gum._proba2color(arcColor[a], cmapArc)
+      if (n, j) in arcColor:
+        col = gumcols.proba2color(arcColor[n, j], cmapArc)
       else:
-        col = gum.getBlackInTheme()
+        col = gumcols.getBlackInTheme()
+
+    dotstr += f' "{bn.variable(n).name()}"->"{bn.variable(j).name()}" [penwidth="{pw}",tooltip="{n}&rarr;{j}:{av}",color="{col}"];'
 
-    dotstr += ' "{0}"->"{1}" [penwidth="{2}",tooltip="{3}:{4}",color="{5}"];'.format(
-      bn.variable(n).name(), bn.variable(j).name(), pw, a, av, col)
   dotstr += '}'
 
-  g = dot.graph_from_dot_data(dotstr)
+  g = dot.graph_from_dot_data(dotstr)[0]
+
+  # workaround for some badly parsed graph (pyparsing>=3.03)
+  g.del_node('"\\n"')
 
   if size is None:
     size = gum.config["notebook", "default_graph_inference_size"]
@@ -268,44 +282,3 @@ def CNinference2dot(cn, size=None, engine=None, evs={}, targets={}, nodeColor=No
   g.temp_dir = temp_dir
 
   return g
-
-
-def dotize(aBN, name, format='pdf'):
-  """
-  From a bn, creates an image of the BN
-
-  :param pyAgrum.BayesNet bn: the bayes net to show
-  :param string name: the filename (without extension) for the image
-  :param string format: format in ['pdf','png','fig','jpg','svg']
-  """
-  if format not in ['pdf', 'png', 'fig', 'jpg', 'svg']:
-    raise Exception(
-      "<%s> in not a correct style ([pdf,png,fig,jpg,svg])" % style)
-
-  if isinstance(aBN, str):
-    bn = gum.loadBN(aBN)
-  else:
-    bn = aBN
-
-  imgfile = name + '.' + format
-  BN2dot(bn).write(imgfile, format=format)
-
-
-def pngize(aBN, name):
-  """
-  From a bn, creates a png of the BN
-
-  :param pyAgrum.BayesNet bn: the bayes net to show
-  :param string name: the filename (without extension) for the image
-  """
-  dotize(aBN, name, 'png')
-
-
-def pdfize(aBN, name):
-  """
-  From a bn, creates a pdf of the BN
-
-  :param pyAgrum.BayesNet bn: the bayes net to show
-  :param string name: the filename (without extension) for the image
-  """
-  dotize(aBN, name, 'pdf')
