@@ -18,7 +18,7 @@
 # IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 # ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
 # OR PERFORMANCE OF THIS SOFTWARE!
-
+from typing import List, Union
 
 import pandas
 import numpy
@@ -35,6 +35,7 @@ from ._utils import _ImplementPrior as IPrior
 from ._utils import _CalculateThreshold as CThreshold
 from ._utils import _DFNames as DFNames
 from ._utils import _createCSVfromNDArrays as CSV
+from ._utils import checkInt
 
 from ._MBCalcul import compileMarkovBlanket
 from ._MBCalcul import _calcul_proba_for_binary_class, _calcul_most_probable_for_nary_class, \
@@ -109,7 +110,7 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
                 also be set to the string 'elbowMethod' so that the best number of bins is found automatically.
                 If the method used is NML, this parameter sets the the maximum number of bins up to which the NML
                 algorithm searches for the optimal number of bins. In this case this parameter must be an int
-                If any other discetization method is used, this parameter is ignored.
+                If any other discretization method is used, this parameter is ignored.
 
             discretizationThreshold: int or float
                 When using default parameters a variable will be treated as continous only if it has more unique values
@@ -269,7 +270,7 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
     # the index of the variable.
     self.variableNameIndexDictionary = None
 
-  def fit(self, X=None, y=None, filename=None, targetName=None):
+  def fit(self, X=None, y=None, data=None, targetName=None,filename=None):
     """
     parameters:
         X: {array-like, sparse matrix} of shape (n_samples, n_features)
@@ -278,52 +279,65 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
         y: array-like of shape (n_samples)
             Target values. Warning: Raises ValueError if either filename or targetname is not None. Raises ValueError
             if X is None
-        filename: str
-            specifies the csv file where the training data and target values are located. Warning: Raises ValueError
-            if either X or y is not None. Raises ValueError if targetName is None
+        data: Union[str,pandas.DataFrame]
+            the source of training data : csv filename or pandas.DataFrame. targetName is mandatory to find the class in this source.
         targetName: str
             specifies the name of the targetVariable in the csv file. Warning: Raises ValueError if either X or y is
             not None. Raises ValueError if filename is None.
+        filename: str
+            (deprecated, use data instead)
+            specifies the csv file where the training data and target values are located. Warning: Raises ValueError
+            if either X or y is not None. Raises ValueError if targetName is None
     returns:
         void
 
-    Fits the model to the training data provided. The two possible uses of this function are fit(X,y) and fit(filename,
-    targetName). Any other combination will raise a ValueError
+    Fits the model to the training data provided. The two possible uses of this function are `fit(X,y)` and `fit(data=...,
+    targetName=...)`. Any other combination will raise a ValueError
     """
+    if filename is not None:
+      print("**pyAgrum** : 'filename' is deprecated since 1.1.1. Please use 'data' instead.")
+      if data is None:
+        data = filename
 
-    if filename is None:
+    if data is None:
       if targetName is not None:
         raise ValueError(
-          "This function should be used either as fit(X,y) or fit(filename=...,targetAttribute=...). You have set "
-          "filename to None, but have entered a targetName")
+          "This function should be used either as fit(X,y) or fit(data=...,targetAttribute=...). You have set "
+          "data to None, but have entered a targetName")
       if X is None or y is None:
         raise ValueError(
-          "This function should be used either as fit(X,y) or fit(filename=...,targetAttribute=...). You have not "
-          "entered a csv file name and not specified the X and y matrices that should be used")
+          "This function should be used either as fit(X,y) or fit(data=...,targetAttribute=...). You have not "
+          "entered a data source (filename or pandas.DataFrame) and not specified the X and y matrices that should be used")
     else:
       if targetName is None:
         raise ValueError(
-          "This function should be used either as fit(X,y) or fit(filename=...,targetAttribute=...). The name of the "
-          "target must be specified if using this function with a csv file.")
+          "This function should be used either as fit(X,y) or fit(data=...,targetAttribute=...). The name of the "
+          "target must be specified if using this function with data  containing a csv filename or a pandas.DataFrame.")
       if X is not None or y is not None:
         raise ValueError(
-          "This function should be used either as fit(X,y) or fit(filename=...,targetAttribute=...). You have entered "
-          "a filename and the X and y matrices at the same time.")
-      X, y = self.XYfromCSV(filename, True, targetName)
+          "This function should be used either as fit(X,y) or fit(data=...,targetAttribute=...). You can not give "
+          "a data and the X and y matrices at the same time.")
+      if type(data)==str:
+        X, y = self.XYfromCSV(data, True, targetName)
+      else: # pandas.DataFrame
+        y = data[targetName]
+        X = data.drop(targetName, axis=1)
 
     self.fromModel = False
     variableNames = None
     self.discretizer.clear()
 
-    if type(y) == pandas.DataFrame:
+    if isinstance(y, pandas.DataFrame):  # type(y) == pandas.DataFrame:
       self.target = y.columns.tolist()[0]
+      if checkInt(self.target):
+        self.target = "Y"
     elif type(y) == pandas.core.series.Series:
       self.target = y.name
     else:
       self.target = 'y'
 
-    if type(X) == pandas.DataFrame:
-      variableNames = X.columns.tolist()
+    if isinstance(X, pandas.DataFrame):  # type(X) == pandas.DataFrame:
+      variableNames = [f"X{x}" if checkInt(x) else x for x in X.columns]
 
     # verifies the shape of the two arrays
     X, y = sklearn.utils.check_X_y(X, y, dtype=None, accept_sparse=True)
@@ -355,8 +369,27 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
     self.bn = gum.BayesNet('Template')
 
     var = gum.LabelizedVariable(self.target, self.target, 0)
+
+    is_int_varY = True
+    min_vY = max_vY = None
     for value in possibleValuesY:
-      var.addLabel(str(value))
+      if not checkInt(value):
+        is_int_varY = False
+        break
+      else:
+        v = int(value)
+        if min_vY is None or min_vY > v:
+          min_vY = v
+        if max_vY is None or max_vY < v:
+          max_vY = v
+
+    if is_int_varY:
+      if len(possibleValuesY) == max_vY - min_vY + 1:  # no hole in the list of int
+        var = gum.RangeVariable(self.target, self.target, min_vY, max_vY)
+      else:
+        var = gum.IntegerVariable(self.target, self.target, [int(v) for v in possibleValuesY])
+    else:
+      var = gum.LabelizedVariable(self.target, self.target, [str(v) for v in possibleValuesY])
     self.bn.add(var)
 
     for i in range(d):
@@ -522,7 +555,7 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
   def predict(self, X, with_labels=True):
     """
     parameters:
-        X: {array-like, sparse matrix} of shape (n_samples, n_features) or str
+        X: str,{array-like, sparse matrix} of shape (n_samples, n_features) or str
             test data, can be either dataFrame, matrix or name of a csv file
         with_labels: bool
             tells us whether the csv includes the labels themselves or their indexes.
@@ -535,7 +568,7 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
     if type(X) == str:
       X, _ = self.XYfromCSV(X, target=self.target)
 
-    if type(X) == pandas.DataFrame:
+    if isinstance(X, pandas.DataFrame):  # type(X) == pandas.DataFrame:
       dictName = DFNames(X)
     else:
       dictName = self.variableNameIndexDictionary
@@ -559,14 +592,24 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
 
     return returned_list
 
-  def _nary_predict(self, X, dictName, with_labels):
+  def _nary_predict(self, X, dictName, with_labels) -> Union[List[str], List[int]]:
     """
-     For a classifier, predicts the most likely class for each row of input data, with bn's Markov Blanket
+    For a classifier, predicts the most likely class for each row of input data, with bn's Markov Blanket
 
-    :param X: data
-    :param dictName: dictionnary of the name of a variable and his column in the data base
-    :param with_labels: tells us whether the csv includes the labels themselves or their indexes.
-    :return:
+    Parameters
+    ----------
+    X: {array-like, sparse matrix} of shape (n_samples, n_features) or str
+            test data, can be either dataFrame, matrix or name of a csv file
+      the data
+    dictName: Dict[str,int]
+      dictionary of the name of a variable and his column in the database
+    with_labels: bool
+      whether `data` contains the labels themselves or their ids.
+
+    Returns
+    -------
+    array-like of shape (n_samples,)
+      the list of predictions as idLabel or label name.
     """
     returned_list = []
     I = self.MarkovBlanket.completeInstantiation()
@@ -576,19 +619,28 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
         x, I, dictName, self.MarkovBlanket, self.target)
       if with_labels:
         returned_list.append(self.MarkovBlanket.variable(
-          self.target).label(vals[0][self.target]))
+          self.target).label(vals[0][0][self.target]))
       else:
-        returned_list.append(vals[0][self.target])
+        returned_list.append(vals[0][0][self.target])
 
     return returned_list
 
-  def _binary_predict(self, X, dictName):
+  def _binary_predict(self, X, dictName) -> Union[List[str], List[bool]]:
     """
-     For a binary classifier, predicts the most likely class for each row of input data, with bn's Markov Blanket
+    For a binary classifier, predicts the most likely class for each row of input data, with bn's Markov Blanket
 
-    :param X: data
-    :param dictName: dictionnary of the name of a variable and his column in the data base
-    :return:
+    Parameters
+    ----------
+    X: {array-like, sparse matrix} of shape (n_samples, n_features) or str
+            test data, can be either dataFrame, matrix or name of a csv file
+      the datas
+    dictName: Dict[str,int]
+      dictionary of the name of a variable and his column in the database
+
+    Returns
+    -------
+    array-like of shape (n_samples,)
+      the list of predictions
     """
     returned_list = []
     # list of other labels of the target
@@ -602,7 +654,7 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
     label1 = self.label
     # Instantiation use to apply values of the data base
     I = self.MarkovBlanket.completeInstantiation()
-    # read through data base's ligns
+    # read through database's ligns
     for x in X:
       res = round(_calcul_proba_for_binary_class(x, label1, labels, I, dictName, self.MarkovBlanket, self.target),
                   self.significant_digit)
@@ -624,20 +676,22 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
 
   def predict_proba(self, X):
     """
-    parameters:
-        X: {array-like, sparse matrix} of shape (n_samples, n_features) or str
-            test data, can be either dataFrame, matrix or name of a csv file
-    returns:
-        y: array-like of shape (n_samples,)
-            Predicted probability for each classes
-
     Predicts the probability of classes for each row of input data, with bn's Markov Blanket
-    """
 
-    # dictionnary of the name of a variable and his column in the data base
+    Parameters
+    ----------
+    X: str or {array-like, sparse matrix} of shape (n_samples, n_features) or str
+            test data, can be either dataFrame, matrix or name of a csv file
+
+    Returns
+    -------
+    array-like of shape (n_samples,)
+      Predicted probability for each classes
+    """
+    # dictionary of the name of a variable and his column in the data base
     dictName = self.variableNameIndexDictionary
 
-    if type(X) == pandas.DataFrame:
+    if isinstance(X, pandas.DataFrame):  # type(X) == pandas.DataFrame:
       dictName = DFNames(X)
       vals = X.to_numpy()
     elif type(X) == str:
@@ -683,23 +737,23 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
 
   def XYfromCSV(self, filename, with_labels=True, target=None):
     """
-    parameters:
-        filename: str
-            the name of the csv file
-        with_labels: bool
-            tells us whether the csv includes the labels themselves or their indexes.
-        target: str or None
-            The name of the column that will be put in the dataframe y. If target is None, we use the target that is
-            already specified in the classifier
-    returns:
-        X: pandas.dataframe
-            Matrix containing the data
-        y: pandas.dataframe
-            Column-vector containing the class for each data vector in X
-
     Reads the data from a csv file and separates it into a X matrix and a y column vector.
-    """
 
+    Parameters
+    ----------
+    filename: str
+        the name of the csv file
+    with_labels: bool
+        tells us whether the csv includes the labels themselves or their indexes.
+    target: str or None
+        The name of the column that will be put in the dataframe y. If target is None, we use the target that is
+        already specified in the classifier
+
+    Returns
+    -------
+    Tuple(pandas.Dataframe,pandas.Dataframe)
+        Matrix X containing the data,Column-vector containing the class for each data vector in X
+    """
     if self.fromModel:
       dataframe = pandas.read_csv(filename, dtype='str')
     else:
@@ -738,16 +792,103 @@ class BNClassifier(sklearn.base.BaseEstimator, sklearn.base.ClassifierMixin):
 
     return X, y
 
+  def preparedData(self, X=None, y=None, data=None, filename=None):
+    """
+    Given an X and a y (or a data source : filename or pandas.DataFrame), returns a pandas.Dataframe with the prepared (especially discretized) values of the base
+
+    Parameters
+    ----------
+        X: {array-like, sparse matrix} of shape (n_samples, n_features)
+            training data. Warning: Raises ValueError if either filename or targetname is not None. Raises ValueError
+            if y is None.
+        y: array-like of shape (n_samples)
+            Target values. Warning: Raises ValueError if either filename or targetname is not None. Raises ValueError
+            if X is None
+        data: Union[str,pandas.DataFrame]
+            specifies the csv file or the DataFrame where the data values are located. Warning: Raises ValueError
+            if either X or y is not None.
+        filename: str
+            (deprecated)
+            specifies the csv file where the data are located. Warning: Raises ValueError
+            if either X or y is not None.
+
+    Returns
+    -------
+      pandas.Dataframe
+    """
+    if self.variableNameIndexDictionary is None:
+      raise ValueError("First, you need to fit a model !")
+    if filename is not None:
+      print("pyAgrum ** : filename is deprecated. Please use data instead.")
+      if data is None:
+        data=filename
+
+    targetName = self.target
+    if data is None:
+      if X is None or y is None:
+        raise ValueError(
+          "This function should be used either as preparedData(X,y) or preparedData(data=...). You have not "
+          "entered a csv file name and not specified the X and y matrices that should be used")
+    else:
+      if targetName is None:
+        raise ValueError(
+          "This function should be used either as preparedData(X,y) or preparedData(data=...). The name of the "
+          "target must be specified if using this function with a csv file.")
+      if X is not None or y is not None:
+        raise ValueError(
+          "This function should be used either as preparedData(X,y) or preparedData(data=...). You have entered "
+          "a filename and the X and y matrices at the same time.")
+
+      if type(data)==str:
+        X, y = self.XYfromCSV(data, True, targetName)
+      else: # pandas.DataFrame
+        y = data[targetName]
+        X = data.drop(targetName, axis=1)
+
+    def bestTypedVal(v, idx):
+      if v.varType() == gum.VarType_Discretized:
+        return v.label(idx)
+      elif v.varType() == gum.VarType_Integer:
+        return int(v.numerical(idx))
+      elif v.varType() == gum.VarType_Labelized:
+        return v.label(idx)
+      elif v.varType() == gum.VarType_Range:
+        return int(v.numerical(idx))
+      else:
+        return None
+
+    reverse = {v: k for k, v in self.variableNameIndexDictionary.items()}
+    if isinstance(X, pandas.DataFrame):  # to be sure of the name of the columns
+      X = X.rename(columns=reverse)
+    varY = self.bn.variable(self.target)
+    df = pandas.DataFrame([], columns=[reverse[k] for k in range(len(reverse))] + [self.target])
+
+    for n in range(len(X)):
+      ligne = []
+      for k in range(len(reverse)):
+        if isinstance(X, pandas.DataFrame):
+          val = X[reverse[k]][n]
+        else:  # np.array
+          val = X[n][k]
+        var = self.bn.variable(reverse[k])
+        ligne.append(bestTypedVal(var, var[str(val)]))
+
+      ligne.append(bestTypedVal(varY, varY[str(y[n])]))
+      df.loc[len(df)] = ligne
+
+    return df
+
   def showROC_PR(self, filename, save_fig=False, show_progress=False):
     """
     Use the `pyAgrum.lib.bn2roc` tools to create ROC and Precision-Recall curve
 
-    parameters:
-      csv_name : str
+    Parameters
+    ----------
+    filename: str
         a csv filename
-      save_fig : bool
-        whether the graph soulb de saved
-      show_progress : bool
+    save_fig : bool
+        whether the graph should be saved
+    show_progress : bool
         indicates if the resulting curve must be printed
     """
     import pyAgrum.lib.bn2roc as bn2roc
